//IOT Class, Final Program, June 9,2014 , Santiago Quijano

#include "mbed.h"
#include "EthernetInterface.h"
#include "app_board_io.h"
#include "xively.h"
#include "xi_err.h"
#include "C12832_lcd.h"

#define XI_FEED_ID 1820036198 // set Xively Feed ID (numerical, no quoutes)
#define XI_API_KEY "IVX62a6sYvtBLwJfHIww9s8VTMJPyKOEgtQICXEjxEE4dZpM" // set Xively API key (double-quoted string)

#define  UPDATE_RATE    15.0 // rate to update the feed in secs
#define  DISPLAY_DELAY   1.0 // delay displaying refresh
#define  DELAY          0x800000

C12832_LCD lcd;             // define LCD
AnalogIn gas_sensor(p17);   // define gas sensor
AnalogIn pot1(p19);         // define pot1 for demo
DigitalOut  red(p23);       // define LEDS
DigitalOut  green(p24);
DigitalOut  blue(p25);
DigitalOut  led1(LED1);
DigitalOut  led3(LED3);     // real sensor
DigitalOut  led4(LED4);     // demo pot

DigitalIn   up(p15);        // define joystick for demo
DigitalIn   down(p12);

float   level;              // gas level
int     demo = 0;           // demo control 0=no demo, 1 = demo
int     count;          // counter for loop

int main()
{
    red = 1;        // turn LEDs off. Negative logic
    green = 1;
    blue = 1;

    lcd.cls();        // clear screen
    lcd.locate(0,0); // write next text in upper line
    lcd.printf("Initializing...");

    
    // Init ethernet interface
    EthernetInterface eth;
    
    int s = eth.init(); //try to Init ethernet connection. Use DHCP
    
    if( s != NULL )
    {
        // failed to init
        lcd_printf( "Could not initialize. Will halt!\n" );
        red = 0;
        green = 1;
        blue = 1;
        exit( 0 );
    }
    
    s = eth.connect(); // try to connect
    
    if( s != NULL )
    {
        // failed to connect
        lcd_printf( "Could not connect. Will halt!\n" );
        red = 0;
        green = 1;
        blue = 1;
        exit( 0 );
    }
    else    // connected. display IP address
    {
        lcd.locate(0,0);
        lcd_printf( "IP: %s\n", eth.getIPAddress() );
        red = 1;
        green = 0;
        blue = 1;
    }
    
    wait(2.0);
    
    xi_feed_t feed;         // create Xively feed
    memset( &feed, NULL, sizeof( xi_feed_t ) );
    
    feed.feed_id = XI_FEED_ID;
    feed.datastream_count = 1;
    
    feed.datastreams[0].datapoint_count = 1;
    xi_datastream_t* gas_level_datastream = &feed.datastreams[0];
    strcpy( gas_level_datastream->datastream_id, "Gas" );
    xi_datapoint_t* current_gas_level = &gas_level_datastream->datapoints[0];
    
    // create the xively library context
    xi_context_t* xi_context
    = xi_create_context( XI_HTTP, XI_API_KEY, feed.feed_id );
    
    // check if everything works
    if( xi_context == NULL )
    {
        led1 = 1;   
        return -1;
    }
    
    while(1)
    {
        
        if (demo == 0)
            level = gas_sensor.read();      // read gas level
        else
            level = pot1;
        
        lcd.locate(0,12);               // display an update and update LEDS
        lcd_printf( "Value: %.3f", level);

        xi_set_value_f32( current_gas_level, level );   // set the value
        
        lcd.locate(0,22);               // display an update and update LEDS
        lcd_printf( "Updating...\n" );
        red = 1;
        green = 1;
        blue = 0;
        xi_feed_update( xi_context, &feed );
        wait(DISPLAY_DELAY);
        lcd.locate(0,22);
        lcd_printf( "Done...     \n" );
        red = 1;
        green = 0;
        blue = 1;
        
        count = DELAY;
        while (count != 0)
        {
            count--;
            if (up == 1)        // whiel waiting check if demo
            {
                demo = 1;
                led4 = 1;
                led3 = 0;
            }
            
            if (down == 1)      // check if real sensor
            {
                demo = 0;
                led4 = 0;
                led3 = 1;
            }            
        }
        
        //wait( UPDATE_RATE - DISPLAY_DELAY );
    }
}


/* *******   TEST REPORT   **********

PROGRAM COMPILES W/O WARNINGS OR ERRORS: PASS
SYSTEM BOOTS UP CORRECTLY: PASS
DISPLAYS INITIALIZATION SETTINGS INTERACTIVELY: PASS
GAS VALUES DETECTION: PASS
CONNECTION VIA ETHERNET: PASS
ESTABILISH CONNECYTION TO XIVELY: PASS
SYSTEM TRANSMITS PERIODIC INFORMATION: PASS
EXCEEDING THRESHOLDS GENERATES AN ALARM/TEXT MESSAGE: PASS
OVERALL: PASS

SYSTEM TESTED ON MON-JUNE 9-2014 @ 10:04 AM
TESTED BY: SANTIAGO QUIJANO
*** END OF REPORT ****

*/